@@ -13,11 +13,13 @@ import (
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/aws/arn"
 	"github.com/aws/aws-sdk-go/aws/awserr"
+	"github.com/aws/aws-sdk-go/aws/client/metadata"
 	"github.com/aws/aws-sdk-go/aws/credentials"
 	"github.com/aws/aws-sdk-go/aws/endpoints"
 	"github.com/aws/aws-sdk-go/aws/request"
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/elb"
+	"github.com/aws/aws-sdk-go/service/elbv2"
 	"github.com/aws/aws-sdk-go/service/resourcegroupstaggingapi"
 	"github.com/aws/aws-sdk-go/service/route53"
 
@@ -26,6 +28,22 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 )
 
+const (
+	// Route53Service is the name of the Route 53 service.
+	Route53Service = route53.ServiceName
+	// ELBService is the name of the Elastic Load Balancing service.
+	ELBService = elb.ServiceName
+	// TaggingService is the name of the Resource Group Tagging service.
+	TaggingService = resourcegroupstaggingapi.ServiceName
+	// govCloudRoute53Region is the AWS GovCloud region for Route 53. See:
+	// https://docs.aws.amazon.com/govcloud-us/latest/UserGuide/using-govcloud-endpoints.html
+	govCloudRoute53Region = "us-gov"
+	// govCloudTaggingEndpoint is the Group Tagging service endpoint used for AWS GovCloud.
+	govCloudTaggingEndpoint = "https://tagging.us-gov-west-1.amazonaws.com"
+	// chinaRoute53Endpoint is the Route 53 service endpoint used for AWS China regions.
+	chinaRoute53Endpoint = "https://route53.amazonaws.com.cn"
+)
+
 var (
 	_   dns.Provider = &Provider{}
 	log              = logf.Logger.WithName("dns")
@@ -40,6 +58,7 @@ var (
 // compared to storing additional metadata (like tags or TXT records).
 type Provider struct {
 	elb     *elb.ELB
+	elbv2   *elbv2.ELBV2
 	route53 *route53.Route53
 	tags    *resourcegroupstaggingapi.ResourceGroupsTaggingAPI
 
@@ -65,6 +84,21 @@ type Config struct {
 	AccessKey string
 	// Region is the AWS region ELBs are created in.
 	Region string
+	// ServiceEndpoints is the list of AWS API endpoints to use for
+	// Provider clients.
+	ServiceEndpoints []ServiceEndpoint
+}
+
+// ServiceEndpoint stores the configuration of a custom url to
+// override existing defaults of AWS Service API endpoints.
+type ServiceEndpoint struct {
+	// name is the name of the AWS service. The full list of service
+	// names can be found at:
+	//   https://docs.aws.amazon.com/general/latest/gr/aws-service-information.html
+	Name string
+	// url is a fully qualified URI that overrides the default generated
+	// AWS API endpoint.
+	URL string
 }
 
 func NewProvider(config Config, operatorReleaseVersion string) (*Provider, error) {
@@ -83,42 +117,114 @@ func NewProvider(config Config, operatorReleaseVersion string) (*Provider, error
 		Fn:   request.MakeAddToUserAgentHandler("openshift.io ingress-operator", operatorReleaseVersion),
 	})
 
-	region := aws.StringValue(sess.Config.Region)
-	if len(region) > 0 {
-		log.Info("using region from shared config", "region name", region)
-	} else {
+	var region string
+	switch {
+	case len(config.Region) > 0:
 		region = config.Region
 		log.Info("using region from operator config", "region name", region)
-	}
-	if len(region) == 0 {
+	case sess.Config.Region != nil:
+		region = aws.StringValue(sess.Config.Region)
+		log.Info("using region from shared config", "region name", region)
+	default:
 		return nil, fmt.Errorf("region is required")
 	}
 
 	r53Config := aws.NewConfig()
-
-	// TODO: The Tagging API will only return hostedzone resources (which are global)
-	// when the region is forced to us-east-1. We don't yet understand why.
-	tagConfig := aws.NewConfig().WithRegion(endpoints.UsEast1RegionID)
+	// elb requires no special region treatment.
+	elbConfig := aws.NewConfig().WithRegion(region)
+	tagConfig := aws.NewConfig()
 
 	// If the region is in aws china, cn-north-1 or cn-northwest-1, we should:
-	// 1. hard code route53 api endpoint to https://route53.amazonaws.com.cn and region to "cn-northwest-1" as route53 is not GA in AWS China,
-	//    and aws sdk didn't have the endpoint.
-	// 2. use the aws china region cn-northwest-1 to setup tagging api correctly in stead of "us-east-1"
+	// 1. hard code route53 api endpoint to https://route53.amazonaws.com.cn and region to "cn-northwest-1"
+	//    as route53 is not GA in AWS China and aws sdk didn't have the endpoint.
+	// 2. use the aws china region cn-northwest-1 to setup tagging api correctly instead of "us-east-1"
 	switch region {
 	case endpoints.CnNorth1RegionID, endpoints.CnNorthwest1RegionID:
 		tagConfig = tagConfig.WithRegion(endpoints.CnNorthwest1RegionID)
-		r53Config = r53Config.WithRegion(endpoints.CnNorthwest1RegionID)
-		r53Config = r53Config.WithEndpoint("https://route53.amazonaws.com.cn")
+		r53Config = r53Config.WithRegion(endpoints.CnNorthwest1RegionID).WithEndpoint(chinaRoute53Endpoint)
+	case endpoints.UsGovEast1RegionID, endpoints.UsGovWest1RegionID:
+		// Route53 for GovCloud uses the "us-gov-west-1" region id:
+		// https://docs.aws.amazon.com/govcloud-us/latest/UserGuide/using-govcloud-endpoints.html
+		r53Config = r53Config.WithRegion(endpoints.UsGovWest1RegionID)
+		// As with other AWS partitions, the GovCloud Tagging client must be
+		// in the same region as the Route53 client to find the hosted zone
+		// of managed records.
+		tagConfig = tagConfig.WithRegion(endpoints.UsGovWest1RegionID)
+	default:
+		// Since Route 53 is not a regionalized service, the Tagging API will
+		// only return hosted zone resources when the region is "us-east-1".
+		tagConfig = tagConfig.WithRegion(endpoints.UsEast1RegionID)
+		// Use us-east-1 for Route 53 in AWS Regions other than China or GovCloud Regions.
+		// See https://docs.aws.amazon.com/general/latest/gr/r53.html for details.
+		r53Config = r53Config.WithRegion(endpoints.UsEast1RegionID)
 	}
-
-	return &Provider{
-		elb:       elb.New(sess, aws.NewConfig().WithRegion(region)),
+	if len(config.ServiceEndpoints) > 0 {
+		route53Found := false
+		elbFound := false
+		tagFound := false
+		for _, ep := range config.ServiceEndpoints {
+			switch {
+			case route53Found && elbFound && tagFound:
+				break
+			case ep.Name == Route53Service:
+				route53Found = true
+				r53Config = r53Config.WithEndpoint(ep.URL)
+				log.Info("using route53 custom endpoint", "url", ep.URL)
+			case ep.Name == TaggingService:
+				tagFound = true
+				url := ep.URL
+				// route53 for govcloud is based out of us-gov-west-1,
+				// so the tagging client must match.
+				if strings.Contains(ep.URL, "us-gov-east-1") {
+					url = govCloudTaggingEndpoint
+				}
+				tagConfig = tagConfig.WithEndpoint(url)
+				log.Info("using group tagging custom endpoint", "url", url)
+			case ep.Name == ELBService:
+				elbFound = true
+				elbConfig = elbConfig.WithEndpoint(ep.URL)
+				log.Info("using elb custom endpoint", "url", ep.URL)
+			}
+		}
+	}
+	p := &Provider{
+		elb: elb.New(sess, elbConfig),
+		// TODO: Add custom endpoint support for elbv2. See the following for details:
+		// https://docs.aws.amazon.com/general/latest/gr/elb.html
+		elbv2:     elbv2.New(sess, aws.NewConfig().WithRegion(region)),
 		route53:   route53.New(sess, r53Config),
 		tags:      resourcegroupstaggingapi.New(sess, tagConfig),
 		config:    config,
 		idsToTags: map[string]map[string]string{},
 		lbZones:   map[string]string{},
-	}, nil
+	}
+	if err := validateServiceEndpoints(p); err != nil {
+		return nil, fmt.Errorf("failed to validate aws provider service endpoints: %v", err)
+	}
+	return p, nil
+}
+
+// validateServiceEndpoints validates that provider clients can communicate with
+// associated API endpoints by having each client make a list/describe/get call.
+func validateServiceEndpoints(provider *Provider) error {
+	var errs []error
+	zoneInput := route53.ListHostedZonesInput{MaxItems: aws.String("1")}
+	if _, err := provider.route53.ListHostedZones(&zoneInput); err != nil {
+		errs = append(errs, fmt.Errorf("failed to list route53 hosted zones: %v", err))
+	}
+	elbInput := elb.DescribeLoadBalancersInput{PageSize: aws.Int64(int64(1))}
+	if _, err := provider.elb.DescribeLoadBalancers(&elbInput); err != nil {
+		errs = append(errs, fmt.Errorf("failed to describe elb load balancers: %v", err))
+	}
+	elbv2Input := elbv2.DescribeLoadBalancersInput{PageSize: aws.Int64(int64(1))}
+	if _, err := provider.elbv2.DescribeLoadBalancers(&elbv2Input); err != nil {
+		errs = append(errs, fmt.Errorf("failed to describe elbv2 load balancers: %v", err))
+	}
+	tagInput := resourcegroupstaggingapi.GetResourcesInput{TagsPerPage: aws.Int64(int64(100))}
+	if _, err := provider.tags.GetResources(&tagInput); err != nil {
+		errs = append(errs, fmt.Errorf("failed to get group tagging resources: %v", err))
+	}
+	return kerrors.NewAggregate(errs)
 }
 
 // getZoneID finds the ID of given zoneConfig in Route53. If an ID is already
@@ -202,11 +308,11 @@ func (m *Provider) getLBHostedZone(name string) (string, error) {
 	}
 
 	var id string
-	fn := func(resp *elb.DescribeLoadBalancersOutput, lastPage bool) (shouldContinue bool) {
+	elbFn := func(resp *elb.DescribeLoadBalancersOutput, lastPage bool) (shouldContinue bool) {
 		for _, lb := range resp.LoadBalancerDescriptions {
 			dnsName := aws.StringValue(lb.DNSName)
 			zoneID := aws.StringValue(lb.CanonicalHostedZoneNameID)
-			log.V(2).Info("found load balancer", "name", aws.StringValue(lb.LoadBalancerName), "dns name", dnsName, "hosted zone ID", zoneID)
+			log.V(2).Info("found classic load balancer", "name", aws.StringValue(lb.LoadBalancerName), "dns name", dnsName, "hosted zone ID", zoneID)
 			if dnsName == name {
 				id = zoneID
 				return false
@@ -214,9 +320,27 @@ func (m *Provider) getLBHostedZone(name string) (string, error) {
 		}
 		return true
 	}
-	err := m.elb.DescribeLoadBalancersPages(&elb.DescribeLoadBalancersInput{}, fn)
+	err := m.elb.DescribeLoadBalancersPages(&elb.DescribeLoadBalancersInput{}, elbFn)
 	if err != nil {
-		return "", fmt.Errorf("failed to describe load balancers: %v", err)
+		return "", fmt.Errorf("failed to describe classic load balancers: %v", err)
+	}
+	if len(id) == 0 {
+		elbv2Fn := func(resp *elbv2.DescribeLoadBalancersOutput, lastPage bool) (shouldContinue bool) {
+			for _, lb := range resp.LoadBalancers {
+				dnsName := aws.StringValue(lb.DNSName)
+				zoneID := aws.StringValue(lb.CanonicalHostedZoneId)
+				log.V(2).Info("found network load balancer", "name", aws.StringValue(lb.LoadBalancerName), "dns name", dnsName, "hosted zone ID", zoneID)
+				if dnsName == name {
+					id = zoneID
+					return false
+				}
+			}
+			return true
+		}
+		err := m.elbv2.DescribeLoadBalancersPages(&elbv2.DescribeLoadBalancersInput{}, elbv2Fn)
+		if err != nil {
+			return "", fmt.Errorf("failed to describe network load balancers: %v", err)
+		}
 	}
 	if len(id) == 0 {
 		return "", fmt.Errorf("couldn't find hosted zone ID of ELB %s", name)
@@ -268,7 +392,7 @@ func (m *Provider) change(record *iov1.DNSRecord, zone configv1.DNSZone, action
 	}
 
 	// Configure records.
-	err = m.updateAlias(domain, zoneID, target, targetHostedZoneID, string(action))
+	err = m.updateRecord(domain, zoneID, target, targetHostedZoneID, string(action), record.Spec.RecordTTL)
 	if err != nil {
 		return fmt.Errorf("failed to update alias in zone %s: %v", zoneID, err)
 	}
@@ -281,18 +405,35 @@ func (m *Provider) change(record *iov1.DNSRecord, zone configv1.DNSZone, action
 	return nil
 }
 
-// updateAlias creates or updates an alias for domain in zoneID pointed at
-// target in targetHostedZoneID.
-func (m *Provider) updateAlias(domain, zoneID, target, targetHostedZoneID, action string) error {
-	resp, err := m.route53.ChangeResourceRecordSets(&route53.ChangeResourceRecordSetsInput{
-		HostedZoneId: aws.String(zoneID),
-		ChangeBatch: &route53.ChangeBatch{
+// updateRecord creates or updates a DNS record for domain in zoneID pointed at
+// target in targetHostedZoneID. An Alias record type is used for all regions
+// other than GovCloud (CNAME). See the following for additional details:
+// https://docs.aws.amazon.com/govcloud-us/latest/UserGuide/govcloud-r53.html
+func (m *Provider) updateRecord(domain, zoneID, target, targetHostedZoneID, action string, ttl int64) error {
+	input := route53.ChangeResourceRecordSetsInput{HostedZoneId: aws.String(zoneID)}
+	if clientEndpointIsGovCloud(&m.route53.Client.ClientInfo) {
+		record := route53.ResourceRecord{Value: aws.String(target)}
+		input.ChangeBatch = &route53.ChangeBatch{
+			Changes: []*route53.Change{
+				{
+					Action: aws.String(action),
+					ResourceRecordSet: &route53.ResourceRecordSet{
+						Name:            aws.String(domain),
+						Type:            aws.String(route53.RRTypeCname),
+						TTL:             aws.Int64(ttl),
+						ResourceRecords: []*route53.ResourceRecord{&record},
+					},
+				},
+			},
+		}
+	} else {
+		input.ChangeBatch = &route53.ChangeBatch{
 			Changes: []*route53.Change{
 				{
 					Action: aws.String(action),
 					ResourceRecordSet: &route53.ResourceRecordSet{
 						Name: aws.String(domain),
-						Type: aws.String("A"),
+						Type: aws.String(route53.RRTypeA),
 						AliasTarget: &route53.AliasTarget{
 							HostedZoneId:         aws.String(targetHostedZoneID),
 							DNSName:              aws.String(target),
@@ -301,8 +442,9 @@ func (m *Provider) updateAlias(domain, zoneID, target, targetHostedZoneID, actio
 					},
 				},
 			},
-		},
-	})
+		}
+	}
+	resp, err := m.route53.ChangeResourceRecordSets(&input)
 	if err != nil {
 		if action == string(deleteAction) {
 			if aerr, ok := err.(awserr.Error); ok {
@@ -317,3 +459,9 @@ func (m *Provider) updateAlias(domain, zoneID, target, targetHostedZoneID, actio
 	log.Info("updated DNS record", "zone id", zoneID, "domain", domain, "target", target, "response", resp)
 	return nil
 }
+
+// clientEndpointIsGovCloud returns true if the provided client info
+// references a US GovCloud API endpoint.
+func clientEndpointIsGovCloud(clientInfo *metadata.ClientInfo) bool {
+	return strings.Contains(clientInfo.Endpoint, govCloudRoute53Region)
+}
