@@ -10,7 +10,6 @@ import (
 	"sort"
 	"strconv"
 	"strings"
-	"time"
 
 	"github.com/davecgh/go-spew/spew"
 	"github.com/google/go-cmp/cmp"
@@ -19,6 +18,7 @@ import (
 	operatorv1 "github.com/openshift/api/operator/v1"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
 	"github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
+	oputil "github.com/openshift/cluster-ingress-operator/pkg/util"
 
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
@@ -36,16 +36,22 @@ import (
 const (
 	WildcardRouteAdmissionPolicy = "ROUTER_ALLOW_WILDCARD_ROUTES"
 
+	RouterForwardedHeadersPolicy = "ROUTER_SET_FORWARDED_HEADERS"
+
+	RouterUniqueHeaderName   = "ROUTER_UNIQUE_ID_HEADER_NAME"
+	RouterUniqueHeaderFormat = "ROUTER_UNIQUE_ID_FORMAT"
+
 	RouterLogLevelEnvName       = "ROUTER_LOG_LEVEL"
 	RouterSyslogAddressEnvName  = "ROUTER_SYSLOG_ADDRESS"
 	RouterSyslogFormatEnvName   = "ROUTER_SYSLOG_FORMAT"
 	RouterSyslogFacilityEnvName = "ROUTER_LOG_FACILITY"
 
+	RouterCaptureHTTPRequestHeaders  = "ROUTER_CAPTURE_HTTP_REQUEST_HEADERS"
+	RouterCaptureHTTPResponseHeaders = "ROUTER_CAPTURE_HTTP_RESPONSE_HEADERS"
+	RouterCaptureHTTPCookies         = "ROUTER_CAPTURE_HTTP_COOKIE"
+
 	RouterDisableHTTP2EnvName          = "ROUTER_DISABLE_HTTP2"
 	RouterDefaultEnableHTTP2Annotation = "ingress.operator.openshift.io/default-enable-http2"
-
-	RouterHardStopAfterEnvName    = "ROUTER_HARD_STOP_AFTER"
-	RouterHardStopAfterAnnotation = "ingress.operator.openshift.io/hard-stop-after"
 )
 
 // ensureRouterDeployment ensures the router deployment exists for a given
@@ -55,7 +61,15 @@ func (r *reconciler) ensureRouterDeployment(ci *operatorv1.IngressController, in
 	if err != nil {
 		return false, nil, err
 	}
-	desired, err := desiredRouterDeployment(ci, r.Config.IngressControllerImage, infraConfig, ingressConfig, apiConfig, networkConfig)
+	platform, err := oputil.GetPlatformStatus(r.client, infraConfig)
+	if err != nil {
+		return false, nil, fmt.Errorf("failed to determine infrastructure platform status for ingresscontroller %s/%s: %v", ci.Namespace, ci.Name, err)
+	}
+	proxyNeeded, err := IsProxyProtocolNeeded(ci, platform)
+	if err != nil {
+		return false, nil, fmt.Errorf("failed to determine if proxy protocol is needed for ingresscontroller %s/%s: %v", ci.Namespace, ci.Name, err)
+	}
+	desired, err := desiredRouterDeployment(ci, r.Config.IngressControllerImage, ingressConfig, apiConfig, networkConfig, proxyNeeded)
 	if err != nil {
 		return haveDepl, current, fmt.Errorf("failed to build router deployment: %v", err)
 	}
@@ -64,12 +78,16 @@ func (r *reconciler) ensureRouterDeployment(ci *operatorv1.IngressController, in
 		if err := r.createRouterDeployment(desired); err != nil {
 			return false, nil, err
 		}
+		return r.currentRouterDeployment(ci)
 	case haveDepl:
-		if err := r.updateRouterDeployment(current, desired); err != nil {
+		if updated, err := r.updateRouterDeployment(current, desired); err != nil {
 			return true, current, err
+		} else if updated {
+			return r.currentRouterDeployment(ci)
 		}
 	}
-	return r.currentRouterDeployment(ci)
+
+	return true, current, nil
 }
 
 // ensureRouterDeleted ensures that any router resources associated with the
@@ -116,35 +134,8 @@ func HTTP2IsEnabled(ic *operatorv1.IngressController, ingressConfig *configv1.In
 	return configHasHTTP2Enabled
 }
 
-// HardStopAfterIsEnabledByAnnotation returns true if the map m has
-// the key RouterHardStopAfterEnvName and its value is a valid HAProxy
-// time duration.
-func HardStopAfterIsEnabledByAnnotation(m map[string]string) (bool, string) {
-	if val, ok := m[RouterHardStopAfterAnnotation]; ok && len(val) > 0 {
-		if clippedVal, err := clipHAProxyTimeoutValue(val); err != nil {
-			log.Error(err, "invalid HAProxy time value", "annotation", RouterHardStopAfterAnnotation, "value", val)
-			return false, ""
-		} else {
-			return true, clippedVal
-		}
-	}
-	return false, ""
-}
-
-// HardStopAfterIsEnabled returns true if either the ingress
-// controller or the ingress config has the "hard-stop-after"
-// annotation. The presence of the annotation on the ingress
-// controller, irrespective of its value, always overrides any setting
-// on the ingress config.
-func HardStopAfterIsEnabled(ic *operatorv1.IngressController, ingressConfig *configv1.Ingress) (bool, string) {
-	if controllerAnnotation, controllerValue := HardStopAfterIsEnabledByAnnotation(ic.Annotations); controllerAnnotation {
-		return controllerAnnotation, controllerValue
-	}
-	return HardStopAfterIsEnabledByAnnotation(ingressConfig.Annotations)
-}
-
 // desiredRouterDeployment returns the desired router deployment.
-func desiredRouterDeployment(ci *operatorv1.IngressController, ingressControllerImage string, infraConfig *configv1.Infrastructure, ingressConfig *configv1.Ingress, apiConfig *configv1.APIServer, networkConfig *configv1.Network) (*appsv1.Deployment, error) {
+func desiredRouterDeployment(ci *operatorv1.IngressController, ingressControllerImage string, ingressConfig *configv1.Ingress, apiConfig *configv1.APIServer, networkConfig *configv1.Network, proxyNeeded bool) (*appsv1.Deployment, error) {
 	deployment := manifests.RouterDeployment()
 	name := controller.RouterDeploymentName(ci)
 	deployment.Name = name.Name
@@ -336,12 +327,8 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		env = append(env, corev1.EnvVar{Name: "ROUTER_CANONICAL_HOSTNAME", Value: ci.Status.Domain})
 	}
 
-	if ci.Status.EndpointPublishingStrategy.Type == operatorv1.LoadBalancerServiceStrategyType {
-		// For now, check if we are on AWS. This can really be done for
-		// for any external [cloud] LBs that support the proxy protocol.
-		if infraConfig.Status.Platform == configv1.AWSPlatformType {
-			env = append(env, corev1.EnvVar{Name: "ROUTER_USE_PROXY_PROTOCOL", Value: "true"})
-		}
+	if proxyNeeded {
+		env = append(env, corev1.EnvVar{Name: "ROUTER_USE_PROXY_PROTOCOL", Value: "true"})
 	}
 
 	env = append(env, corev1.EnvVar{Name: "ROUTER_THREADS", Value: "4"})
@@ -405,91 +392,119 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 	secretName := controller.RouterEffectiveDefaultCertificateSecretName(ci, deployment.Namespace)
 	deployment.Spec.Template.Spec.Volumes[0].Secret.SecretName = secretName.Name
 
-	accessLogging := accessLoggingForIngressController(ci)
-	switch {
-	case accessLogging == nil:
-		// No additional configuration is needed.
-	case accessLogging.Destination.Type == operatorv1.ContainerLoggingDestinationType:
-		rsyslogConfigVolume := corev1.Volume{
-			Name: "rsyslog-config",
-			VolumeSource: corev1.VolumeSource{
-				ConfigMap: &corev1.ConfigMapVolumeSource{
-					LocalObjectReference: corev1.LocalObjectReference{
-						Name: controller.RsyslogConfigMapName(ci).Name,
+	if accessLogging := accessLoggingForIngressController(ci); accessLogging != nil {
+		switch {
+		case accessLogging.Destination.Type == operatorv1.ContainerLoggingDestinationType:
+			rsyslogConfigVolume := corev1.Volume{
+				Name: "rsyslog-config",
+				VolumeSource: corev1.VolumeSource{
+					ConfigMap: &corev1.ConfigMapVolumeSource{
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: controller.RsyslogConfigMapName(ci).Name,
+						},
 					},
 				},
-			},
-		}
-		rsyslogConfigVolumeMount := corev1.VolumeMount{
-			Name:      rsyslogConfigVolume.Name,
-			MountPath: "/etc/rsyslog",
-		}
+			}
+			rsyslogConfigVolumeMount := corev1.VolumeMount{
+				Name:      rsyslogConfigVolume.Name,
+				MountPath: "/etc/rsyslog",
+			}
 
-		// Ideally we would use a Unix domain socket in the abstract
-		// namespace, but rsyslog does not support that, so we need a
-		// filesystem that is common to the router and syslog
-		// containers.
-		rsyslogSocketVolume := corev1.Volume{
-			Name: "rsyslog-socket",
-			VolumeSource: corev1.VolumeSource{
-				EmptyDir: &corev1.EmptyDirVolumeSource{},
-			},
-		}
-		rsyslogSocketVolumeMount := corev1.VolumeMount{
-			Name:      rsyslogSocketVolume.Name,
-			MountPath: "/var/lib/rsyslog",
-		}
-
-		configPath := filepath.Join(rsyslogConfigVolumeMount.MountPath, "rsyslog.conf")
-		socketPath := filepath.Join(rsyslogSocketVolumeMount.MountPath, "rsyslog.sock")
-
-		syslogContainer := corev1.Container{
-			Name: operatorv1.ContainerLoggingSidecarContainerName,
-			// The ingresscontroller image has rsyslog built in.
-			Image: ingressControllerImage,
-			Command: []string{
-				"/sbin/rsyslogd", "-n",
-				// TODO: Once we have rsyslog 8.32 or later,
-				// we can switch to -i NONE.
-				"-i", "/tmp/rsyslog.pid",
-				"-f", configPath,
-			},
-			ImagePullPolicy: corev1.PullIfNotPresent,
-			VolumeMounts: []corev1.VolumeMount{
-				rsyslogConfigVolumeMount,
-				rsyslogSocketVolumeMount,
-			},
-			Resources: corev1.ResourceRequirements{
-				Requests: corev1.ResourceList{
-					corev1.ResourceCPU:    resource.MustParse("100m"),
-					corev1.ResourceMemory: resource.MustParse("256Mi"),
+			// Ideally we would use a Unix domain socket in the abstract
+			// namespace, but rsyslog does not support that, so we need a
+			// filesystem that is common to the router and syslog
+			// containers.
+			rsyslogSocketVolume := corev1.Volume{
+				Name: "rsyslog-socket",
+				VolumeSource: corev1.VolumeSource{
+					EmptyDir: &corev1.EmptyDirVolumeSource{},
 				},
-			},
+			}
+			rsyslogSocketVolumeMount := corev1.VolumeMount{
+				Name:      rsyslogSocketVolume.Name,
+				MountPath: "/var/lib/rsyslog",
+			}
+
+			configPath := filepath.Join(rsyslogConfigVolumeMount.MountPath, "rsyslog.conf")
+			socketPath := filepath.Join(rsyslogSocketVolumeMount.MountPath, "rsyslog.sock")
+
+			syslogContainer := corev1.Container{
+				Name: operatorv1.ContainerLoggingSidecarContainerName,
+				// The ingresscontroller image has rsyslog built in.
+				Image: ingressControllerImage,
+				Command: []string{
+					"/sbin/rsyslogd", "-n",
+					// TODO: Once we have rsyslog 8.32 or later,
+					// we can switch to -i NONE.
+					"-i", "/tmp/rsyslog.pid",
+					"-f", configPath,
+				},
+				ImagePullPolicy: corev1.PullIfNotPresent,
+				VolumeMounts: []corev1.VolumeMount{
+					rsyslogConfigVolumeMount,
+					rsyslogSocketVolumeMount,
+				},
+				Resources: corev1.ResourceRequirements{
+					Requests: corev1.ResourceList{
+						corev1.ResourceCPU:    resource.MustParse("100m"),
+						corev1.ResourceMemory: resource.MustParse("256Mi"),
+					},
+				},
+			}
+
+			env = append(env,
+				corev1.EnvVar{Name: RouterSyslogAddressEnvName, Value: socketPath},
+				corev1.EnvVar{Name: RouterLogLevelEnvName, Value: "debug"},
+			)
+			volumes = append(volumes, rsyslogConfigVolume, rsyslogSocketVolume)
+			routerVolumeMounts = append(routerVolumeMounts, rsyslogSocketVolumeMount)
+			deployment.Spec.Template.Spec.Containers = append(deployment.Spec.Template.Spec.Containers, syslogContainer)
+		case accessLogging.Destination.Type == operatorv1.SyslogLoggingDestinationType:
+			if len(accessLogging.Destination.Syslog.Facility) > 0 {
+				env = append(env, corev1.EnvVar{Name: RouterSyslogFacilityEnvName, Value: accessLogging.Destination.Syslog.Facility})
+			}
+			address := accessLogging.Destination.Syslog.Address
+			port := accessLogging.Destination.Syslog.Port
+			endpoint := net.JoinHostPort(address, fmt.Sprintf("%d", port))
+			env = append(env,
+				corev1.EnvVar{Name: RouterLogLevelEnvName, Value: "debug"},
+				corev1.EnvVar{Name: RouterSyslogAddressEnvName, Value: endpoint},
+			)
 		}
 
-		env = append(env,
-			corev1.EnvVar{Name: RouterSyslogAddressEnvName, Value: socketPath},
-			corev1.EnvVar{Name: RouterLogLevelEnvName, Value: "debug"},
-		)
 		if len(accessLogging.HttpLogFormat) > 0 {
 			env = append(env, corev1.EnvVar{Name: RouterSyslogFormatEnvName, Value: fmt.Sprintf("%q", accessLogging.HttpLogFormat)})
 		}
-		volumes = append(volumes, rsyslogConfigVolume, rsyslogSocketVolume)
-		routerVolumeMounts = append(routerVolumeMounts, rsyslogSocketVolumeMount)
-		deployment.Spec.Template.Spec.Containers = append(deployment.Spec.Template.Spec.Containers, syslogContainer)
-	case accessLogging.Destination.Type == operatorv1.SyslogLoggingDestinationType:
-		if len(accessLogging.Destination.Syslog.Facility) > 0 {
-			env = append(env, corev1.EnvVar{Name: RouterSyslogFacilityEnvName, Value: accessLogging.Destination.Syslog.Facility})
+		if val := serializeCaptureHeaders(accessLogging.HTTPCaptureHeaders.Request); len(val) != 0 {
+			env = append(env, corev1.EnvVar{
+				Name:  RouterCaptureHTTPRequestHeaders,
+				Value: val,
+			})
 		}
-		address := accessLogging.Destination.Syslog.Address
-		port := accessLogging.Destination.Syslog.Port
-		endpoint := net.JoinHostPort(address, fmt.Sprintf("%d", port))
-		env = append(env,
-			corev1.EnvVar{Name: RouterLogLevelEnvName, Value: "debug"},
-			corev1.EnvVar{Name: RouterSyslogAddressEnvName, Value: endpoint},
-		)
-		if len(accessLogging.HttpLogFormat) > 0 {
-			env = append(env, corev1.EnvVar{Name: RouterSyslogFormatEnvName, Value: fmt.Sprintf("%q", accessLogging.HttpLogFormat)})
+		if val := serializeCaptureHeaders(accessLogging.HTTPCaptureHeaders.Response); len(val) != 0 {
+			env = append(env, corev1.EnvVar{
+				Name:  RouterCaptureHTTPResponseHeaders,
+				Value: val,
+			})
+		}
+		if len(accessLogging.HTTPCaptureCookies) > 0 {
+			var (
+				cookieName string
+				maxLength  = accessLogging.HTTPCaptureCookies[0].MaxLength
+			)
+			switch accessLogging.HTTPCaptureCookies[0].MatchType {
+			case operatorv1.CookieMatchTypeExact:
+				cookieName = accessLogging.HTTPCaptureCookies[0].Name + "="
+			case operatorv1.CookieMatchTypePrefix:
+				cookieName = accessLogging.HTTPCaptureCookies[0].NamePrefix
+			}
+			if maxLength == 0 {
+				maxLength = 256
+			}
+			env = append(env, corev1.EnvVar{
+				Name:  RouterCaptureHTTPCookies,
+				Value: fmt.Sprintf("%s:%d", cookieName, maxLength),
+			})
 		}
 	}
 
@@ -560,16 +575,41 @@ func desiredRouterDeployment(ci *operatorv1.IngressController, ingressController
 		env = append(env, corev1.EnvVar{Name: WildcardRouteAdmissionPolicy, Value: "false"})
 	}
 
+	forwardedHeaderPolicy := operatorv1.AppendHTTPHeaderPolicy
+	if ci.Spec.HTTPHeaders != nil && len(ci.Spec.HTTPHeaders.ForwardedHeaderPolicy) != 0 {
+		forwardedHeaderPolicy = ci.Spec.HTTPHeaders.ForwardedHeaderPolicy
+	}
+	routerForwardedHeadersPolicyValue := "append"
+	switch forwardedHeaderPolicy {
+	case operatorv1.AppendHTTPHeaderPolicy:
+		// Nothing to do.
+	case operatorv1.ReplaceHTTPHeaderPolicy:
+		routerForwardedHeadersPolicyValue = "replace"
+	case operatorv1.IfNoneHTTPHeaderPolicy:
+		routerForwardedHeadersPolicyValue = "if-none"
+	case operatorv1.NeverHTTPHeaderPolicy:
+		routerForwardedHeadersPolicyValue = "never"
+	}
+	env = append(env, corev1.EnvVar{Name: RouterForwardedHeadersPolicy, Value: routerForwardedHeadersPolicyValue})
+
+	if ci.Spec.HTTPHeaders != nil && len(ci.Spec.HTTPHeaders.UniqueId.Name) > 0 {
+		headerName := ci.Spec.HTTPHeaders.UniqueId.Name
+		headerFormat := ci.Spec.HTTPHeaders.UniqueId.Format
+		if len(headerFormat) == 0 {
+			headerFormat = "%{+X}o %ci:%cp_%fi:%fp_%Ts_%rt:%pid"
+		}
+		env = append(env,
+			corev1.EnvVar{Name: RouterUniqueHeaderName, Value: headerName},
+			corev1.EnvVar{Name: RouterUniqueHeaderFormat, Value: fmt.Sprintf("%q", headerFormat)},
+		)
+	}
+
 	if HTTP2IsEnabled(ci, ingressConfig) {
 		env = append(env, corev1.EnvVar{Name: RouterDisableHTTP2EnvName, Value: "false"})
 	} else {
 		env = append(env, corev1.EnvVar{Name: RouterDisableHTTP2EnvName, Value: "true"})
 	}
 
-	if enabled, value := HardStopAfterIsEnabled(ci, ingressConfig); enabled {
-		env = append(env, corev1.EnvVar{Name: RouterHardStopAfterEnvName, Value: value})
-	}
-
 	deployment.Spec.Template.Spec.Volumes = volumes
 	deployment.Spec.Template.Spec.Containers[0].VolumeMounts = routerVolumeMounts
 	deployment.Spec.Template.Spec.Containers[0].Env = append(deployment.Spec.Template.Spec.Containers[0].Env, env...)
@@ -603,7 +643,9 @@ func accessLoggingForIngressController(ic *operatorv1.IngressController) *operat
 				Type:      operatorv1.ContainerLoggingDestinationType,
 				Container: &operatorv1.ContainerLoggingDestinationParameters{},
 			},
-			HttpLogFormat: ic.Spec.Logging.Access.HttpLogFormat,
+			HttpLogFormat:      ic.Spec.Logging.Access.HttpLogFormat,
+			HTTPCaptureHeaders: ic.Spec.Logging.Access.HTTPCaptureHeaders,
+			HTTPCaptureCookies: ic.Spec.Logging.Access.HTTPCaptureCookies,
 		}
 	case operatorv1.SyslogLoggingDestinationType:
 		if ic.Spec.Logging.Access.Destination.Syslog != nil {
@@ -616,13 +658,26 @@ func accessLoggingForIngressController(ic *operatorv1.IngressController) *operat
 						Facility: ic.Spec.Logging.Access.Destination.Syslog.Facility,
 					},
 				},
-				HttpLogFormat: ic.Spec.Logging.Access.HttpLogFormat,
+				HttpLogFormat:      ic.Spec.Logging.Access.HttpLogFormat,
+				HTTPCaptureHeaders: ic.Spec.Logging.Access.HTTPCaptureHeaders,
+				HTTPCaptureCookies: ic.Spec.Logging.Access.HTTPCaptureCookies,
 			}
 		}
 	}
 	return nil
 }
 
+// serializeCaptureHeaders serializes a slice of
+// IngressControllerCaptureHTTPHeader values into a value suitable for
+// ROUTER_CAPTURE_HTTP_RESPONSE_HEADERS or ROUTER_CAPTURE_HTTP_REQUEST_HEADERS.
+func serializeCaptureHeaders(captureHeaders []operatorv1.IngressControllerCaptureHTTPHeader) string {
+	var headerSpecs []string
+	for _, header := range captureHeaders {
+		headerSpecs = append(headerSpecs, fmt.Sprintf("%s:%d", header.Name, header.MaxLength))
+	}
+	return strings.Join(headerSpecs, ",")
+}
+
 // inferTLSProfileSpecFromDeployment examines the given deployment's pod
 // template spec and reconstructs a TLS profile spec based on that pod spec.
 func inferTLSProfileSpecFromDeployment(deployment *appsv1.Deployment) *configv1.TLSProfileSpec {
@@ -903,19 +958,19 @@ func (r *reconciler) createRouterDeployment(deployment *appsv1.Deployment) error
 }
 
 // updateRouterDeployment updates a router deployment.
-func (r *reconciler) updateRouterDeployment(current, desired *appsv1.Deployment) error {
+func (r *reconciler) updateRouterDeployment(current, desired *appsv1.Deployment) (bool, error) {
 	changed, updated := deploymentConfigChanged(current, desired)
 	if !changed {
-		return nil
+		return false, nil
 	}
 
 	// Diff before updating because the client may mutate the object.
 	diff := cmp.Diff(current, updated, cmpopts.EquateEmpty())
 	if err := r.client.Update(context.TODO(), updated); err != nil {
-		return fmt.Errorf("failed to update router deployment %s/%s: %v", updated.Namespace, updated.Name, err)
+		return false, fmt.Errorf("failed to update router deployment %s/%s: %v", updated.Namespace, updated.Name, err)
 	}
 	log.Info("updated router deployment", "namespace", updated.Namespace, "name", updated.Name, "diff", diff)
-	return nil
+	return true, nil
 }
 
 // deepHashObject writes specified object to hash using the spew library
@@ -972,46 +1027,3 @@ func deploymentConfigChanged(current, expected *appsv1.Deployment) (bool, *appsv
 	updated.Spec.Replicas = &replicas
 	return true, updated
 }
-
-// clipHAProxyTimeoutValue prevents the HAProxy config file from using
-// timeout values that exceed the maximum value allowed by HAProxy.
-// Returns an error in the event that a timeout string value is not
-// parsable as a valid time duration, or the clipped time duration
-// otherwise.
-//
-// TODO: this is a modified copy from openshift/router but returns ""
-// if there's any error.
-//
-// Ideally we need to share this utility function via:
-// https://github.com/openshift/library-go/blob/master/pkg/route/routeapihelpers
-func clipHAProxyTimeoutValue(val string) (string, error) {
-	const haproxyMaxTimeout = "2147483647ms" // max timeout allowable by HAProxy
-
-	endIndex := len(val) - 1
-	maxTimeout, err := time.ParseDuration(haproxyMaxTimeout)
-	if err != nil {
-		return "", err
-	}
-	// time.ParseDuration doesn't work with days
-	// despite HAProxy accepting timeouts that specify day units
-	if val[endIndex] == 'd' {
-		days, err := strconv.Atoi(val[:endIndex])
-		if err != nil {
-			return "", err
-		}
-		if maxTimeout.Hours() < float64(days*24) {
-			log.V(7).Info("Route annotation timeout exceeds maximum allowable by HAProxy, clipping to max")
-			return haproxyMaxTimeout, nil
-		}
-	} else {
-		duration, err := time.ParseDuration(val)
-		if err != nil {
-			return "", err
-		}
-		if maxTimeout.Milliseconds() < duration.Milliseconds() {
-			log.V(7).Info("Route annotation timeout exceeds maximum allowable by HAProxy, clipping to max")
-			return haproxyMaxTimeout, nil
-		}
-	}
-	return val, nil
-}
